Title VFOOD.ASM
;******************************************************************************
;
;By Keith Jin, Microsoft PSS, July 16, 1991
;
;	This code shows the basic structure of a VxD. It does nothing except
;	output debug messages to the debug terminal. Please note, the Trace_Out
;	and Debug_Out macros are only assemblyed when the 'DEBUG' switch is
;	turned on.
;
;==============================================================================

	.386p

	.xlist
	include vmm.inc
	include debug.inc
	include vfood.inc
	.list

VFOOD_CB_STRUCT STRUC
VM_Birth_Time	dd	?
VFOOD_CB_STRUCT ENDS

;******************************************************************************
; declare virtual device
;	The Declare_Virtual_Device macro creates a structure of data called
;	Device Data Block (DDB). The name is this data block is <VxD name>_DDB.
;	For this VxD, it will be VFOOD_DDB. This name must be exported in the
;	.def file. (See VFOOD.DEF). This data structure is the only place that
;	VMM will find various information about the VxD. See Virtual Device
;	Adaptation Guide for more information. If this VxD exports services,
;	you don't need to include the service table name in this macro. The
;	service table just needs to be declared using the same VxD name. See
;	VFOOD.INC.
;
;	See VFOOD.INC for information on VFOOD_Device_ID.
;==============================================================================

Declare_Virtual_Device VFOOD, 1, 0, VFOOD_Control, VFOOD_DEVICE_ID, \
		       Undefined_Init_Order, VFOOD_V86_API,VFOOD_PM_API

;******************************************************************************
; data
;==============================================================================
VXD_DATA_SEG
	public	VFOOD_CB_Offset
VFOOD_CB_Offset 	dd	0	;

VXD_DATA_ENDS

;******************************************************************************
; init stuff
;==============================================================================

VXD_IDATA_SEG

; (some data used during initialization time)

VXD_IDATA_ENDS

VXD_ICODE_SEG		; Please see code and comments in VFOOD_Control first.

;******************************************************************************
;
;   VFOOD_Sys_Crit_Init
;
;   DESCRIPTION: Called during the first phase of PM init.
;
;   ENTRY: EBX: Sys VM handle
;
;   EXIT: carry flag clear.
;
;   USES:
;
;==============================================================================
BeginProc VFOOD_Sys_Crit_Init

	Trace_Out "VFOOD: Sys Critical Init."

	VMMCall _Allocate_Device_CB_Area, <<SIZE VFOOD_CB_STRUCT>, 0>
	or	eax, eax
	jz	short VFOOD_SCI_Error
	mov	[VFOOD_CB_Offset], eax
	add	eax, ebx
	mov	[eax.VM_Birth_Time], 0
	clc
	ret

VFOOD_SCI_Error:
;
; Debug_Out prints the message and breaks into the debugger.
;
	Debug_Out "VFOOD: Fail to allocate VM CB area."
	VMMjmp	Fatal_Memory_Error

EndProc VFOOD_Sys_Crit_Init

;******************************************************************************
;
;   VFOOD_Device_Init
;
;   DESCRIPTION: Called during the second phase of PM init.
;
;   ENTRY: EBX: Sys VM handle
;
;   EXIT:
;
;   USES:
;
;==============================================================================
BeginProc VFOOD_Device_Init

	Trace_Out "VFOOD: Device Init."
	ret

EndProc VFOOD_Device_Init

;******************************************************************************
;
;   VFOOD_Init_Complete
;
;   DESCRIPTION: Called during the final phase of PM init.
;
;   ENTRY: EBX: Sys VM handle
;
;   EXIT:
;
;   USES:
;
;==============================================================================
BeginProc VFOOD_Init_Complete

	Trace_Out "VFOOD: Init Complete"
	ret

EndProc VFOOD_Init_Complete

VXD_ICODE_ENDS

;******************************************************************************
; Code
;------------------------------------------------------------------------------
VXD_CODE_SEG

;******************************************************************************
;
;   VFOOD_Control
;
;   DESCRIPTION: The system (or VMM) calls this entry point for various events
;	that is occuring in the system, for example, system init, system exit,
;	VM create, VM init, or VM being destroyed... Here are some of the
;	"messages." There are others not used here. This proc dispatches the
;	call to the appropriate routines. Control_Dispatch is a macro. Please
;	see VMM.INC for more detailed information.
;==============================================================================
BeginProc VFOOD_Control

	Control_Dispatch Sys_Critical_Init, VFOOD_Sys_Crit_Init
	Control_Dispatch Device_Init, VFOOD_Device_Init
	Control_Dispatch Init_Complete, VFOOD_Init_Complete
	Control_Dispatch Create_VM, <SHORT VFOOD_Create_VM>
	Control_Dispatch VM_Init, <SHORT VFOOD_VM_Init>
	Control_Dispatch Destroy_VM, <SHORT VFOOD_Destroy_VM>
	Control_Dispatch Set_Device_Focus, VFOOD_Set_Focus
	Control_Dispatch System_Exit, VFOOD_System_Exit
	clc	      ; don't care about others for now
	ret

EndProc VFOOD_Control

;******************************************************************************
; Create VM
;==============================================================================

BeginProc VFOOD_Create_VM

	Trace_Out "VFOOD: VM #EBX is being created."

	mov	edi, [VFOOD_CB_Offset]
	add	edi, ebx
	VMMCall Get_System_Time
	mov	[edi.VM_Birth_Time], eax
	clc
	ret

EndProc VFOOD_Create_VM

;******************************************************************************
; VM init
;==============================================================================
BeginProc VFOOD_VM_Init

	Trace_Out "VFOOD: VM #EBX init"
	ret

EndProc VFOOD_VM_Init

;******************************************************************************
; VM Terminate
;==============================================================================
BeginProc VFOOD_VM_Terminate

	Trace_Out "VFOOD: VM #EBX terminates."
	ret

EndProc VFOOD_VM_Terminate

;******************************************************************************
; VM Not Executable
;==============================================================================
BeginProc VFOOD_VM_Not_Executeable

	Trace_Out "VFOOD: VM #EBX not execteable"
	ret

EndProc VFOOD_VM_Not_Executeable

;******************************************************************************
; VM Destroy
;==============================================================================
BeginProc VFOOD_Destroy_VM

	mov	edi, [VFOOD_CB_Offset]
	add	edi, ebx
	VMMCall Get_System_Time
	sub	eax, [edi.VM_Birth_Time]
	Trace_Out "VFOOD: VM #EBX Destroyed. It was around for #eax milliseconds."
	ret

EndProc VFOOD_Destroy_VM

;******************************************************************************
;VFOOD_Set_Focus
;==============================================================================

BeginProc VFOOD_Set_Focus

	mov	eax, edx
	Trace_out "VFOOD: Set Device #EAX Focus to VM #EBX"
	ret

EndProc VFOOD_Set_Focus

;******************************************************************************
;VFOOD_System_Exit
;==============================================================================
BeginProc VFOOD_System_Exit

	Trace_Out "Well, have a good life!"
	ret

EndProc VFOOD_System_Exit

;******************************************************************************
; VFOOD_Get_Version
;	The only VFOOD service provided in this code. VxD services can be
;	called by other VxDs. For example, "VxDCall VFOOD_Get_Version"
;==============================================================================
BeginProc VFOOD_Get_Version, Service

	Trace_Out "VFOOD: Service get version."
	mov	eax, 100h
	ret

EndProc VFOOD_Get_Version
;******************************************************************************
;VFOOD_V86_API
;	Entry point for V86 API. This procedure can be a dispatch point for
;	different functions of the API.
;
;Entry: EBX: Current VM handle
;	EBP -> Client Register Struct
;
;Exit: Client_DX:AX = VM's birth time
;==============================================================================

BeginProc VFOOD_V86_API

	Trace_Out "VFOOD: V86 API call from VM #EBX"
	mov	edi, [VFOOD_CB_Offset]
	add	edi, ebx
	mov	eax, [edi.VM_Birth_Time]
	mov	[ebp.Client_AX], ax
	shr	eax, 16
	mov	[ebp.Client_DX], ax
	ret

EndProc VFOOD_V86_API

;******************************************************************************
;VFOOD_PM_API
;	Entry point for PM API. This procedure can be a dispatch point for
;	different functions of the API.
;
;Entry: EBX: Current VM handle
;	EBP -> Client Register Struct
;Exit: VFOOD version number in Client_DX:AX
;==============================================================================

BeginProc VFOOD_PM_API
	Trace_Out "VFOOD: PM API call from VM #EBX"
	VxDCall VFOOD_Get_Version		; other VxDs can do the same.
	mov	[ebp.Client_AX], ax
	shr	eax, 16
	mov	[ebp.Client_DX], ax
	ret
EndProc VFOOD_PM_API

VXD_CODE_ENDS

	END
